// Define the company hierarchy data
const data = {
  name: 'CEO',
  children: [
    {
      name: 'Engineering Department',
      shape: 'circle',
      color: 'darkblue',
      children: [
        {
          name: 'Development',
          shape: 'rect',
          color: 'darkred',
          children: [
            {
              name: 'Frontend Team',
              shape: 'rect',
              color: 'red',
              children: [
                {
                  name: 'Alice',
                  shape: 'polygon',
                  color: 'green',
                },
                {
                  name: 'Bob',
                  shape: 'polygon',
                  color: 'green',
                },
              ],
            },
            {
              name: 'Data Team',
              shape: 'rect',
              color: 'red',
              children: [
                {
                  name: 'Matt',
                  shape: 'polygon',
                  color: 'green',
                },
                {
                  name: 'Smith',
                  shape: 'polygon',
                  color: 'green',
                },
              ],
            },
            {
              name: 'Backend Team',
              shape: 'rect',
              color: 'red',
              children: [
                {
                  name: 'Charlie',
                  shape: 'polygon',
                  color: 'green',
                },
                {
                  name: 'David',
                  shape: 'polygon',
                  color: 'green',
                },
              ],
            },
          ],
        },
        {
          name: 'Testing',
          shape: 'rect',
          color: 'darkred',
          children: [
            {
              name: 'QA Team',
              shape: 'rect',
              color: 'red',
              children: [
                {
                  name: 'Eve',
                  shape: 'polygon',
                  color: 'green',
                },
                {
                  name: 'Frank',
                  shape: 'polygon',
                  color: 'green',
                },
              ],
            },
          ],
        },
      ],
    },
    {
      name: 'Marketing Department',
      shape: 'circle',
      color: 'darkblue',
      children: [
        {
          name: 'Advertising',
          shape: 'rect',
          color: 'darkred',
          children: [
            {
              name: 'Creative Team',
              shape: 'rect',
              color: 'red',
              children: [
                // Corrected placement of children array
                {
                  name: 'Grace',
                  shape: 'polygon',
                  color: 'green',
                },
                {
                  name: 'John',
                  shape: 'polygon',
                  color: 'green',
                },
              ],
            },
            {
              name: 'Digital Marketing Team',
              shape: 'rect',
              color: 'red',
              children: [
                // Corrected placement of children array
                {
                  name: 'Adam',
                  shape: 'polygon',
                  color: 'green',
                },
                {
                  name: 'Emily',
                  shape: 'polygon',
                  color: 'green',
                },
              ],
            },
          ],
        },
        {
          name: 'Sales',
          shape: 'rect',
          color: 'darkred',
          children: [
            {
              name: 'East Region Team',
              shape: 'rect',
              color: 'red',
              children: [
                {
                  name: 'Michael',
                  shape: 'polygon',
                  color: 'green',
                },
                {
                  name: 'Sophia',
                  shape: 'polygon',
                  color: 'green',
                },
              ],
            },
            {
              name: 'West Region Team',
              shape: 'rect',
              color: 'red',
              children: [
                {
                  name: 'William',
                  shape: 'polygon',
                  color: 'green',
                },
                {
                  name: 'Olivia',
                  shape: 'polygon',
                  color: 'green',
                },
              ],
            },
          ],
        },
      ],
    },
  ],
};

// Define width, height, and margin of the tree
const margin = {
  top: 10,
  right: 250,
  bottom: 200,
  left: 200,
};
const width = 850 - margin.left - margin.right;
const height = 650 - margin.top - margin.bottom;

const tree = d3.tree().size([height, width]);

// Assign hierarchical structure to classify the data
let nodes = d3.hierarchy(data, (node) => node.children);

// Provide data connection for nodes to their respective position in the tree
nodes = tree(nodes);

// Check if SVG already exists
const svgExists = d3.select('svg').empty();

// Add an SVG element if it doesn't exist
if (svgExists) {
  const svg = d3
    .select('body')
    .append('svg')
    .attr('width', width + margin.left + margin.right)
    .attr('height', height + margin.top + margin.bottom);
  const g = svg
    .append('g') // append group
    .attr(
      'transform',
      'translate(' + margin.left + ',' + margin.top + ')',
    );

  // Define links between nodes
  const link = g
    .selectAll('.link')
    .data(nodes.descendants().slice(1))
    .enter()
    .append('path')
    .attr('class', 'link')
    .style('stroke', (d) => d.data.color)
    .attr('d', (d) => {
      return (
        'M' +
        d.y +
        ',' +
        d.x +
        'C' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.x +
        ' ' +
        (d.y + d.parent.y) / 2 +
        ',' +
        d.parent.x +
        ' ' +
        d.parent.y +
        ',' +
        d.parent.x
      );
    });

  // Declare a group for each node present in the tree
  const node = g
    .selectAll('.node')
    .data(nodes.descendants())
    .enter()
    .append('g')
    .attr(
      'class',
      (d) =>
        'node' +
        (d.children ? ' node--internal' : ' node--leaf'),
    )
    .attr(
      'transform',
      (d) => 'translate(' + d.y + ',' + d.x + ')',
    );

  // Define shape of the node
  node.each(function (d) {
    const shape = d.data.shape;
    if (shape === 'circle') {
      d3.select(this)
        .append('circle')
        .attr('r', 10) // radius of circle
        .style('stroke', (d) => d.data.color)
        .style('fill');
    } else if (shape === 'rect') {
      d3.select(this)
        .append('rect')
        .attr('width', 50) // width of rectangle
        .attr('height', 20) // height of rectangle
        .attr('rx', 5) // corner radius
        .style('stroke', (d) => d.data.color)
        .style('fill');
    } else if (shape === 'polygon') {
      const points = '0,10 5,0 10,10'; // points for diamond shape
      d3.select(this)
        .append('polygon')
        .attr('points', points)
        .style('stroke', (d) => d.data.color)
        .style('fill');
    }
  });

  // Add text to nodes
  node
    .append('text')
    .attr('dy', '.60em')
    .attr('x', (d) => (d.children ? -15 : 15))
    .attr('y', (d) =>
      d.children && d.depth !== 0 ? -15 : 0,
    )
    .style('text-anchor', (d) =>
      d.children ? 'end' : 'start',
    )
    .text((d) => d.data.name);
}
